-- =====================================================================
-- Copyright © 2010-2012 by Cryptographic Engineering Research Group (CERG),
-- ECE Department, George Mason University
-- Fairfax, VA, U.S.A.
-- =====================================================================

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all; 
use work.sha3_pkg.all;
use work.sha3_jh_package.all;

entity jh_control_mem is		
	generic ( 
		h : integer := 256
	);
	port (					
		rst						: in std_logic;
		clk						: in std_logic;
		
		-- datapath signals
			--fsm1
		ec, lc					: out std_logic;
		ein, li					: out std_logic;
		wmin1, wmin2, wmin3, wmin4 : out std_logic;
		din_addr				: out std_logic_vector(log2(b/fbw)-1 downto 0);
		zc0, final_segment		: in std_logic;
		
			--fsm2
		sf, srdp, srin 			: out std_logic;
		srout1, srout2			: out std_logic;				
		round 					: out std_logic_vector(log2(64*b/fbw)-1 downto 0);
		r_addr1_1, r_addr2_1	: out std_logic_vector(log2(b/fbw)-1 downto 0); 
		r_addr1_2, r_addr2_2	: out std_logic_vector(log2(b/fbw)-1 downto 0);
		whp, wr1, wr2			: out std_logic;
			-- fsm3
		sdout				 	: out std_logic_vector(h/256-1 downto 0);
		eout					: out std_logic;			
		
		-- fifo signals
		src_ready				: in std_logic;
		src_read				: out std_logic;
		dst_ready 				: in std_logic;
		dst_write				: out std_logic
	);				 
end jh_control_mem;

architecture struct of jh_control_mem is				   	 
	-- fsm1
	signal fsm1_addr : std_logic_vector(log2(b/fbw)-1 downto 0);
	-- fsm2						 
	signal fsm2_addr1_1, fsm2_addr2_1 : std_logic_vector(log2(b/fbw)-1 downto 0); 
	signal fsm2_srout1, fsm2_srout2 : std_logic;

	-- fsm3
	signal fsm3_addr : std_logic_vector(log2(b/fbw)-1 downto 0);
	signal fsm3_srout1, fsm3_srout2 : std_logic;

	-- sync sigs					
	signal msg_ready, msg_end, hash_processed, hash_complete, load_next_block : std_logic;
begin
	
	fsm1_gen : entity work.jh_fsm1(beh) 
		port map (
		clk => clk, rst => rst, 
		ein => ein, li => li, ec => ec, lc => lc, zc0 => zc0, final_segment => final_segment,
		load_next_block => load_next_block, msg_ready => msg_ready, msg_end => msg_end,
		src_ready => src_ready, src_read => src_read,
		addr => fsm1_addr, w1 => wmin1, w2 => wmin2, w3 => wmin3, w4 => wmin4
	);	 
	
	fsm2_gen : entity work.jh_fsm2(beh) 
	port map (
		clk => clk, rst => rst, 
		sf => sf, srdp => srdp,
		round => round, wr1 => wr1, wr2 => wr2, whp => whp, 
		srin => srin, srout1 => fsm2_srout1, srout2 => fsm2_srout2,
		r_addr1_1 => fsm2_addr1_1, r_addr1_2 => r_addr1_2, r_addr2_1 => fsm2_addr2_1, r_addr2_2 => r_addr2_2,
		msg_ready => msg_ready,	msg_end => msg_end, hash_processed => hash_processed, 
		load_next_block	=> load_next_block, hash_complete => hash_complete
	); 
	
	fsm3_gen : entity work.jh_fsm3(beh) 
		generic map ( h => h )
		port map (
		clk => clk, rst => rst, 
		sdout => sdout, srout1 => fsm3_srout1, srout2 => fsm3_srout2,
		eout => eout, r_addr => fsm3_addr, msg_end => hash_processed, hash_complete => hash_complete,
		dst_ready => dst_ready,	dst_write => dst_write
	);	 	

	din_addr <= fsm2_addr1_1 when msg_ready = '1' else fsm1_addr;
	r_addr1_1 <= fsm3_addr when hash_processed = '1' else fsm2_addr1_1;
	r_addr2_1 <= fsm3_addr when hash_processed = '1' else fsm2_addr2_1;
	srout1 <= fsm3_srout1 when hash_processed = '1' else fsm2_srout1;
	srout2 <= fsm3_srout2 when hash_processed = '1' else fsm2_srout2;
	

end struct;
