-- =====================================================================
-- Copyright © 2010-2012 by Cryptographic Engineering Research Group (CERG),
-- ECE Department, George Mason University
-- Fairfax, VA, U.S.A.
-- =====================================================================

-- Possible generics values: 
--		hs = {HASH_SIZE_256, HASH_SIZE_512} 

library ieee;
use ieee.std_logic_1164.all; 
use work.sha3_pkg.all;
use work.sha3_jh_package.all;

entity jh_top is		
	generic (		
		hs : integer := HASH_SIZE_256
	); 
	port (		
		-- global
		rst 	: in std_logic;
		clk 	: in std_logic;
		
		--fifo
		src_ready : in std_logic;
		src_read  : out std_logic;
		dst_ready : in std_logic;
		dst_write : out std_logic;		
		din		: in std_logic_vector(w-1 downto 0);
		dout	: out std_logic_vector(w-1 downto 0)
	);	   
end jh_top;


architecture structure of jh_top is 
	-- fsm1
	signal lc, ec, li, ein : std_logic;
	signal wmin1, wmin2, wmin3, wmin4 : std_logic;
	signal din_addr : std_logic_vector(log2(b/fbw)-1 downto 0);
	signal zc0 : std_logic;
	-- fsm2
	signal round : std_logic_vector(log2(64*b/fbw)-1 downto 0);
	signal r_addr1_1, r_addr1_2, r_addr2_1, r_addr2_2 : std_logic_vector(log2(b/fbw)-1 downto 0);
	signal whp, wr1, wr2 : std_logic;
	signal sf, srdp, srin, srout1, srout2 : std_logic;
	-- fsm3
	signal sdout : std_logic_vector(hs/256-1 downto 0); 
	signal eout : std_logic;
	-- top fsm

begin					
	control : entity work.jh_control_mem(struct)
			generic map ( h => hs )
			port map (
			rst => rst, clk => clk,
			src_ready => src_ready, src_read => src_read, dst_ready => dst_ready, dst_write => dst_write,
			wmin1 => wmin1, wmin2 => wmin2, wmin3 => wmin3, wmin4 => wmin4,
			final_segment => din(w-1), din_addr => din_addr, 
			zc0 => zc0, ec => ec, lc => lc, li => li, ein => ein,
			r_addr1_1 => r_addr1_1, r_addr2_1 => r_addr2_1, r_addr1_2 => r_addr1_2, r_addr2_2 => r_addr2_2,
			sf => sf, srdp => srdp, srin => srin, srout1 => srout1, srout2 => srout2,
			whp => whp, wr1 => wr1, wr2 => wr2, round => round,	 
			sdout => sdout, eout => eout
		);			
		
	datapath : entity work.jh_datapath_mem(struct) 
			generic map ( h => hs )
			port map (
			clk => clk, din => din, dout => dout,
			wmin1 => wmin1, wmin2 => wmin2, wmin3 => wmin3, wmin4 => wmin4,
			din_addr => din_addr, zc0 => zc0, ec => ec, lc => lc, li => li, ein => ein,
			r_addr1_1 => r_addr1_1, r_addr2_1 => r_addr2_1, r_addr1_2 => r_addr1_2, r_addr2_2 => r_addr2_2,
			sf => sf, srdp => srdp, srin => srin, srout1 => srout1, srout2 => srout2,
			whp => whp, wr1 => wr1, wr2 => wr2, round => round,	 
			sdout => sdout, eout => eout
		);		  

	
end structure;
	
	