-- =====================================================================
-- Copyright © 2010-2012 by Cryptographic Engineering Research Group (CERG),
-- ECE Department, George Mason University
-- Fairfax, VA, U.S.A.
-- =====================================================================

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_unsigned.all;
use work.sha3_pkg.all;

-- RAM

entity ramn is
	generic(
		M : INTEGER := 64;
		N : INTEGER := 8
		);
	port(
		input : in STD_LOGIC_VECTOR(M-1 DOWNTO 0);
		addr : in STD_LOGIC_VECTOR(log2(N)-1 DOWNTO 0);
		write : in STD_LOGIC;
		clk : in STD_LOGIC;
		output : out STD_LOGIC_VECTOR(M-1 DOWNTO 0)
	    );
end ramn;

architecture behavioral of ramn is
	type ram is array( 0 to N-1 ) of std_logic_vector(M-1 downto 0);
	signal data : ram;
begin
	PROCESS (clk)
	BEGIN
		IF rising_edge(clk) THEN
			IF write = '1'THEN
				data(to_integer(unsigned(addr))) <= input;
			END IF;
		END IF;
	END PROCESS;
	output <= data(to_integer(unsigned(addr)));
end behavioral;
